//
// by S. Gehlbach
// edit with :set ts=4
// common routines that can be inlined
//
// WARNING: all value -> register & pci routines have been coded
//          to standard AT&T direction, ie, out(value,reg) etc.
//
#define COMMON

#define PCI_DEVFN(slot,func)    ((((slot) & 0x1f) << 3) | ((func) & 0x07))
#define PCI_SLOT(devfn)         (((devfn) >> 3) & 0x1f)
#define PCI_FUNC(devfn)         ((devfn) & 0x07)

#define NORTH_BRIDGE_BUS 0
#define NORTH_BRIDGE_DEV 0
#define NORTH_BRIDGE_FUN 0
#define NORTH_BRIDGE_DEVFN PCI_DEVFN(NORTH_BRIDGE_DEV,NORTH_BRIDGE_FUN);

#define LPC_BRIDGE_BUS 0
#define LPC_BRIDGE_DEV 1
#define LPC_BRIDGE_FUN  0
#define LPC_BRIDGE_DEVFN PCI_DEVFN(LPC_BRIDGE_DEV,LPC_BRIDGE_FUN);

#define PCI_PCI_BRIDGE_BUS 0
#define PCI_PCI_BRIDGE_DEV 2
#define PCI_PCI_BRIDGE_FUN 0
#define PCI_PCI_BRIDGE_DEVFN PCI_DEVFN(PCI_PCI_BRIDGE_DEV,PCI_PCI_BRIDGE_FUN);

#define CONFIG_CMD(bus,devfn, where)   (0x80000000 | (bus << 16) | (devfn << 8) | where )

extern inline void outb (unsigned char value, unsigned short port) {
	    asm volatile ("outb %b0,%w1": :"a" (value), "Nd" (port));
}

extern inline void outl (unsigned int value, unsigned short port) {
	    asm volatile ("outl %0,%w1": :"a" (value), "Nd" (port));
}

extern inline unsigned char inb (unsigned short port) {
	unsigned char _v asm ("al");
	asm volatile ("inb %w1,%0":"=a" (_v):"Nd" (port));
	return _v;
}

static inline void pci_conf1_write_config_byte(unsigned char value, int addr) {
        outl(addr & ~3, 0xCF8);
        outb(value, 0xCFC + (addr & 3));
}

static inline unsigned char pci_conf1_read_config_byte(int addr) {
        outl(addr & ~3, 0xCF8);
        return inb(0xCFC + (addr & 3));
}
